# Generated by roxygen2: do not edit by hand

S3method("$",matrixset)
S3method("[",col_grouped_ms)
S3method("[",dual_grouped_ms)
S3method("[",matrixset)
S3method("[",row_grouped_ms)
S3method("[<-",matrixset)
S3method("[[",matrixset)
S3method("column_info<-",col_grouped_ms)
S3method("column_info<-",default)
S3method("column_info<-",dual_grouped_ms)
S3method("column_info<-",matrixset)
S3method("column_traits<-",col_grouped_ms)
S3method("column_traits<-",default)
S3method("column_traits<-",dual_grouped_ms)
S3method("column_traits<-",matrixset)
S3method("dimnames<-",matrixset)
S3method("matrix_elm<-",default)
S3method("matrix_elm<-",matrixset)
S3method("matrixnames<-",default)
S3method("matrixnames<-",matrixset)
S3method("row_info<-",default)
S3method("row_info<-",dual_grouped_ms)
S3method("row_info<-",matrixset)
S3method("row_info<-",row_grouped_ms)
S3method("row_traits<-",default)
S3method("row_traits<-",dual_grouped_ms)
S3method("row_traits<-",matrixset)
S3method("row_traits<-",row_grouped_ms)
S3method(add_matrix,matrixset)
S3method(annotate_column,matrixset)
S3method(annotate_row,matrixset)
S3method(apply_column,col_grouped_ms)
S3method(apply_column,dual_grouped_ms)
S3method(apply_column,matrixset)
S3method(apply_column,row_grouped_ms)
S3method(apply_column_dfl,col_grouped_ms)
S3method(apply_column_dfl,dual_grouped_ms)
S3method(apply_column_dfl,matrixset)
S3method(apply_column_dfl,row_grouped_ms)
S3method(apply_column_dfw,col_grouped_ms)
S3method(apply_column_dfw,dual_grouped_ms)
S3method(apply_column_dfw,matrixset)
S3method(apply_column_dfw,row_grouped_ms)
S3method(apply_matrix,col_grouped_ms)
S3method(apply_matrix,dual_grouped_ms)
S3method(apply_matrix,matrixset)
S3method(apply_matrix,row_grouped_ms)
S3method(apply_matrix_dfl,col_grouped_ms)
S3method(apply_matrix_dfl,dual_grouped_ms)
S3method(apply_matrix_dfl,matrixset)
S3method(apply_matrix_dfl,row_grouped_ms)
S3method(apply_matrix_dfw,col_grouped_ms)
S3method(apply_matrix_dfw,dual_grouped_ms)
S3method(apply_matrix_dfw,matrixset)
S3method(apply_matrix_dfw,row_grouped_ms)
S3method(apply_row,col_grouped_ms)
S3method(apply_row,dual_grouped_ms)
S3method(apply_row,matrixset)
S3method(apply_row,row_grouped_ms)
S3method(apply_row_dfl,col_grouped_ms)
S3method(apply_row_dfl,dual_grouped_ms)
S3method(apply_row_dfl,matrixset)
S3method(apply_row_dfl,row_grouped_ms)
S3method(apply_row_dfw,col_grouped_ms)
S3method(apply_row_dfw,dual_grouped_ms)
S3method(apply_row_dfw,matrixset)
S3method(apply_row_dfw,row_grouped_ms)
S3method(arrange_column,matrixset)
S3method(arrange_row,matrixset)
S3method(as_matrixset,default)
S3method(as_matrixset,list)
S3method(as_matrixset,matrix)
S3method(column_group_by,matrixset)
S3method(column_group_by_drop_default,matrixset)
S3method(column_group_indices,col_grouped_ms)
S3method(column_group_indices,dual_grouped_ms)
S3method(column_group_indices,matrixset)
S3method(column_group_keys,col_grouped_ms)
S3method(column_group_keys,dual_grouped_ms)
S3method(column_group_keys,matrixset)
S3method(column_group_meta,col_grouped_ms)
S3method(column_group_meta,dual_grouped_ms)
S3method(column_group_meta,matrixset)
S3method(column_group_vars,col_grouped_ms)
S3method(column_group_vars,dual_grouped_ms)
S3method(column_group_vars,matrixset)
S3method(column_group_where,col_grouped_ms)
S3method(column_group_where,dual_grouped_ms)
S3method(column_group_where,matrixset)
S3method(column_groups,col_grouped_ms)
S3method(column_groups,dual_grouped_ms)
S3method(column_groups,matrixset)
S3method(column_info,default)
S3method(column_info,matrixset)
S3method(column_tag,default)
S3method(column_tag,matrixset)
S3method(column_traits,default)
S3method(column_traits,matrixset)
S3method(column_ungroup,col_grouped_ms)
S3method(column_ungroup,dual_grouped_ms)
S3method(column_ungroup,matrixset)
S3method(column_ungroup,row_grouped_ms)
S3method(dim,matrixset)
S3method(dimnames,matrixset)
S3method(filter_column,col_grouped_ms)
S3method(filter_column,dual_grouped_ms)
S3method(filter_column,matrixset)
S3method(filter_row,dual_grouped_ms)
S3method(filter_row,matrixset)
S3method(filter_row,row_grouped_ms)
S3method(join_column_info,matrixset)
S3method(join_row_info,matrixset)
S3method(matrix_elm,default)
S3method(matrix_elm,matrixset)
S3method(matrixnames,default)
S3method(matrixnames,matrixset)
S3method(ms_to_df,matrixset)
S3method(mutate_matrix,matrixset)
S3method(nmatrix,default)
S3method(nmatrix,matrixset)
S3method(print,matrixset)
S3method(remove_column_annotation,matrixset)
S3method(remove_matrix,matrixset)
S3method(remove_row_annotation,matrixset)
S3method(row_group_by,matrixset)
S3method(row_group_by_drop_default,matrixset)
S3method(row_group_indices,dual_grouped_ms)
S3method(row_group_indices,matrixset)
S3method(row_group_indices,row_grouped_ms)
S3method(row_group_keys,dual_grouped_ms)
S3method(row_group_keys,matrixset)
S3method(row_group_keys,row_grouped_ms)
S3method(row_group_meta,dual_grouped_ms)
S3method(row_group_meta,matrixset)
S3method(row_group_meta,row_grouped_ms)
S3method(row_group_vars,dual_grouped_ms)
S3method(row_group_vars,matrixset)
S3method(row_group_vars,row_grouped_ms)
S3method(row_group_where,dual_grouped_ms)
S3method(row_group_where,matrixset)
S3method(row_group_where,row_grouped_ms)
S3method(row_groups,dual_grouped_ms)
S3method(row_groups,matrixset)
S3method(row_groups,row_grouped_ms)
S3method(row_info,default)
S3method(row_info,matrixset)
S3method(row_tag,default)
S3method(row_tag,matrixset)
S3method(row_traits,default)
S3method(row_traits,matrixset)
S3method(row_ungroup,col_grouped_ms)
S3method(row_ungroup,dual_grouped_ms)
S3method(row_ungroup,matrixset)
S3method(row_ungroup,row_grouped_ms)
export("column_info<-")
export("column_traits<-")
export("matrix_elm<-")
export("matrixnames<-")
export("row_info<-")
export("row_traits<-")
export(add_matrix)
export(annotate_column)
export(annotate_column_from_apply)
export(annotate_row)
export(annotate_row_from_apply)
export(apply_column)
export(apply_column_dfl)
export(apply_column_dfw)
export(apply_matrix)
export(apply_matrix_dfl)
export(apply_matrix_dfw)
export(apply_row)
export(apply_row_dfl)
export(apply_row_dfw)
export(arrange_column)
export(arrange_row)
export(as_matrixset)
export(column_group_by)
export(column_group_by_drop_default)
export(column_group_indices)
export(column_group_keys)
export(column_group_meta)
export(column_group_vars)
export(column_group_where)
export(column_groups)
export(column_info)
export(column_pos)
export(column_rel_pos)
export(column_tag)
export(column_traits)
export(column_ungroup)
export(current_column_info)
export(current_column_name)
export(current_n_column)
export(current_n_row)
export(current_row_info)
export(current_row_name)
export(filter_column)
export(filter_row)
export(is_matrixset)
export(join_column_info)
export(join_row_info)
export(matrix_elm)
export(matrixnames)
export(matrixset)
export(ms_to_df)
export(mutate_matrix)
export(nmatrix)
export(remove_column_annotation)
export(remove_matrix)
export(remove_row_annotation)
export(row_group_by)
export(row_group_by_drop_default)
export(row_group_indices)
export(row_group_keys)
export(row_group_meta)
export(row_group_vars)
export(row_group_where)
export(row_groups)
export(row_info)
export(row_pos)
export(row_rel_pos)
export(row_tag)
export(row_traits)
export(row_ungroup)
importFrom(R6,R6Class)
importFrom(Rcpp,sourceCpp)
importFrom(methods,is)
importFrom(rlang,.data)
importFrom(stats,setNames)
useDynLib(matrixset, .registration = TRUE)
